//Author: Ian Goodfellow
//File: svlISMCodebook.h
//Description: A class for learning and applying an Implicit Shape Model

#pragma once

#include "svlVision.h"
#include "svlDevel.h"
#include "svlISMpatch.h"

enum svlISMPhaseType
  {
    ACCUM,
    COUNT
  };


class svlOccurrance
{
 public:

  svlOccurrance() {}
 svlOccurrance(const CvPoint & p, unsigned f) : x(p.x), y(p.y), frame(f) {}
  
  int x;
  int y;
  unsigned frame;
};

class svlISM
{
 public:
 
  svlISM(unsigned int width, unsigned int height, unsigned int numChannels, double ngcThreshold, svlInterestPointDetector * ipd = NULL); //If ipd is non-null, user is responsible for freeing it
  svlISM(const char * filename);
  ~svlISM();

  void setShowInterestPointsDuringAccum(bool setting) {  _showInterestPointsAccum = setting; }

  void learnPatches(const vector<IplImage *> & channels);
  //void agglomerate();

  void finishLearningPatches();

  void learnOffsets(const vector<IplImage *> & channels, const svlObject2dFrame & labels);

  svlObject2dFrame getDetections(const vector<IplImage *> & channels);
  svlObject2dFrame testScaleVote(const vector<IplImage *> & channels);

  bool read(const char *filename);
  bool write(const char *filename) const;
 

  bool operator==(const svlISM & o) const;

  unsigned numChannels() const { return _numChannels; }
  unsigned width() const { return _width; }
  unsigned height() const { return _height; }
  unsigned numPatches(unsigned ch) const { return _patches[ch].size(); }
  svlISMPhaseType phase() const { return _phase; }
  svlISMpatch * getPatch(unsigned ch, unsigned idx) { return _patches[ch][idx]; }

  void debugPrune(); //cuts the model down to just one patch so that it is easier to debug other functions


 protected:

  void appendPatches(vector<svlISMpatch *> & output, const IplImage * img, const vector<CvPoint> & points) const;
  svlISMpatch * getPatch(const IplImage * img, const CvPoint & pt) const; //todo-- this is done with pixel ops, could be sped up
  void addPatch(svlISMpatch * p, vector<svlISMpatch *> & dictionary);
  void getPatchesAndLocations(const IplImage * img, vector<svlISMpatch *> & impatches, vector<CvPoint> & locations) const;

  void destroyPatches();

 protected:


  //The dimensions of the patches used in the codebook
  unsigned int _width;
  unsigned int _height;
  //These are not truly the half widths. they are the number of pixels on one side of the center (ie, floor of half)
  unsigned int _halfWidth;
  unsigned int _halfHeight;

  unsigned _numChannels;

  svlISMPhaseType _phase; //How far along in the training process our model is: if ACCUM, then we are still adding patches. If COUNT, we have already agglomerated the patches and are now adding examples of objects related to each patch

  svlInterestPointDetector * _ipd; //Interest point detector used to determine which patches to extract
  bool _ownIpd; //If true, it is our responsibility to free _ipd

  vector<vector<svlISMpatch *> > _patches; //_patches[i][j] is patch j of the dictionary for channel i
  vector<vector<vector<CvPoint> > > _offsets; //_offsets[i][j][k] is the distance to the object centroid for the k'th time that a feature on the object activated _patches[i][j]
  vector<vector<unsigned> > _negatives; //_negatives[i][j] is the number of times that patch j in channel i has appeared and not been on an object


  vector<svlObject2dFrame> _objects;//todo-- rename this to frames
  vector<vector<vector<svlOccurrance> > > _occurrances;


  //The average NGC between the clusters in two patches must be above
  //this threshold for the clusters to be merged
  //The NGC between a patch in an image and a patch in the dictionary
  //must be above the same threshold for the image to activate that
  //dictionary entry
  double _ngcThreshold;  


  vector<CvPoint> _votes;
  vector<CvPoint> _voteScales;
  vector<double> _voteWeights;
  vector<CvPoint> _voteSources;


  bool _showInterestPointsAccum;
};
