//Author: Ian Goodfellow
//File: svlInterestPointDetector.h
//Description: A pure virtual class describing the methods
// an interest point detector should provide. Other classes
// can then implement these methods with various methods such
// as Harris corner detection, SIFT features, etc.

#pragma once

#include "cv.h"
#include "cxcore.h"
#include <fstream>
#include <vector>
#include "xmlParser/xmlParser.h"
using namespace std;

class svlInterestPointDetector
{
 public:
  virtual ~svlInterestPointDetector() {}

  virtual void findInterestPoints(const IplImage * img, vector<CvPoint> & output) = 0;

  virtual void write(ofstream & ofs) const = 0;

  static svlInterestPointDetector * readXMLNode(XMLNode & node);

  virtual bool operator!=(const svlInterestPointDetector & other) const = 0;
};

